@@ -8,10 +8,12 @@ import re
 import shlex
 import string
 import sys
+import traceback
 import warnings
 from collections import OrderedDict
 from fnmatch import fnmatchcase
 from subprocess import list2cmdline
+from threading import Thread
 
 import pkg_resources
 import pluggy
@@ -28,6 +30,7 @@ from tox.reporter import (
     using,
     verbosity1,
 )
+from tox.util.path import ensure_empty_dir
 
 from .parallel import ENV_VAR_KEY as PARALLEL_ENV_VAR_KEY
 from .parallel import add_parallel_config, add_parallel_flags
@@ -45,6 +48,8 @@ Instead create a hookimpl in your code with:
 default_factors = tox.PYTHON.DEFAULT_FACTORS
 """DEPRECATED MOVE - please update to new location."""
 
+WITHIN_PROVISION = os.environ.get(str("TOX_PROVISION")) == "1"
+
 
 def get_plugin_manager(plugins=()):
     # initialize plugin manager
@@ -114,8 +119,11 @@ class Parser:
         assert hasattr(obj, "postprocess")
         self._testenv_attr.append(obj)
 
-    def parse_cli(self, args):
-        return self.argparser.parse_args(args)
+    def parse_cli(self, args, strict=False):
+        args, argv = self.argparser.parse_known_args(args)
+        if argv and (strict or WITHIN_PROVISION):
+            self.argparser.error("unrecognized arguments: %s".format(" ".join(argv)))
+        return args
 
     def _format_help(self):
         return self.argparser.format_help()
@@ -384,7 +392,8 @@ def tox_addoption(parser):
     parser.add_argument(
         "--showconfig",
         action="store_true",
-        help="show configuration information for all environments. ",
+        help="show live configuration (by default all env, with -l only default targets,"
+        " specific via TOXENV/-e)",
     )
     parser.add_argument(
         "-l",
@@ -744,6 +753,14 @@ def tox_addoption(parser):
         "have access to globally installed packages.",
     )
 
+    parser.add_testenv_attribute(
+        "download",
+        type="bool",
+        default=False,
+        help="download the latest pip, setuptools and wheel when creating the virtual"
+        "environment (default is to use the one bundled in virtualenv)",
+    )
+
     parser.add_testenv_attribute(
         name="alwayscopy",
         type="bool",
@@ -1023,6 +1040,9 @@ class ParseIni(object):
         config.sdistsrc = reader.getpath("sdistsrc", None)
         config.setupdir = reader.getpath("setupdir", "{toxinidir}")
         config.logdir = config.toxworkdir.join("log")
+        within_parallel = PARALLEL_ENV_VAR_KEY in os.environ
+        if not within_parallel:
+            ensure_empty_dir(config.logdir)
 
         # determine indexserver dictionary
         config.indexserver = {"default": IndexServerConfig("default")}
@@ -1058,7 +1078,8 @@ class ParseIni(object):
         self.handle_provision(config, reader)
 
         self.parse_build_isolation(config, reader)
-        config.envlist, all_envs = self._getenvdata(reader, config)
+        res = self._getenvdata(reader, config)
+        config.envlist, all_envs, config.envlist_default, config.envlist_explicit = res
 
         # factors used in config or predefined
         known_factors = self._list_section_factors("testenv")
@@ -1071,6 +1092,18 @@ class ParseIni(object):
                 known_factors.update(env.split("-"))
 
         # configure testenvs
+        to_do = []
+        failures = OrderedDict()
+        results = {}
+        cur_self = self
+
+        def run(name, section, subs, config):
+            try:
+                results[name] = cur_self.make_envconfig(name, section, subs, config)
+            except Exception as exception:
+                failures[name] = (exception, traceback.format_exc())
+
+        order = []
         for name in all_envs:
             section = "{}{}".format(testenvprefix, name)
             factors = set(name.split("-"))
@@ -1081,7 +1114,23 @@ class ParseIni(object):
                     tox.PYTHON.PY_FACTORS_RE.match(factor) for factor in factors - known_factors
                 )
             ):
-                config.envconfigs[name] = self.make_envconfig(name, section, reader._subs, config)
+                order.append(name)
+                thread = Thread(target=run, args=(name, section, reader._subs, config))
+                thread.daemon = True
+                thread.start()
+                to_do.append(thread)
+        for thread in to_do:
+            while thread.is_alive():
+                thread.join(timeout=20)
+        if failures:
+            raise tox.exception.ConfigError(
+                "\n".join(
+                    "{} failed with {} at {}".format(key, exc, trace)
+                    for key, (exc, trace) in failures.items()
+                )
+            )
+        for name in order:
+            config.envconfigs[name] = results[name]
         all_develop = all(
             name in config.envconfigs and config.envconfigs[name].usedevelop
             for name in config.envlist
@@ -1106,6 +1155,9 @@ class ParseIni(object):
             env_config.deps = deps
             config.envconfigs[config.provision_tox_env] = env_config
             raise tox.exception.MissingRequirement(config)
+        # if provisioning is not on, now we need do a strict argument evaluation
+        # raise on unknown args
+        self.config._parser.parse_cli(args=self.config.args, strict=True)
 
     @staticmethod
     def ensure_requires_satisfied(config, requires, min_version):
@@ -1218,18 +1270,19 @@ class ParseIni(object):
         from_config = reader.getstring("envlist", replace=False)
 
         env_list = []
+        envlist_explicit = False
         if (from_option and "ALL" in from_option) or (
             not from_option and from_environ and "ALL" in from_environ.split(",")
         ):
             all_envs = self._getallenvs(reader)
         else:
             candidates = (
-                os.environ.get(PARALLEL_ENV_VAR_KEY),
-                from_option,
-                from_environ,
-                from_config,
+                (os.environ.get(PARALLEL_ENV_VAR_KEY), True),
+                (from_option, True),
+                (from_environ, True),
+                (from_config, False),
             )
-            env_str = next((i for i in candidates if i), [])
+            env_str, envlist_explicit = next(((i, e) for i, e in candidates if i), ([], False))
             env_list = _split_env(env_str)
             all_envs = self._getallenvs(reader, env_list)
 
@@ -1243,7 +1296,7 @@ class ParseIni(object):
         if config.isolated_build is True and package_env in env_list:
             msg = "isolated_build_env {} cannot be part of envlist".format(package_env)
             raise tox.exception.ConfigError(msg)
-        return env_list, all_envs
+        return env_list, all_envs, _split_env(from_config), envlist_explicit
 
 
 def _split_env(env):
@@ -1303,21 +1356,22 @@ class DepConfig:
         self.name = name
         self.indexserver = indexserver
 
-    def __str__(self):
+    def __repr__(self):
         if self.indexserver:
             if self.indexserver.name == "default":
                 return self.name
             return ":{}:{}".format(self.indexserver.name, self.name)
         return str(self.name)
 
-    __repr__ = __str__
-
 
 class IndexServerConfig:
     def __init__(self, name, url=None):
         self.name = name
         self.url = url
 
+    def __repr__(self):
+        return "IndexServerConfig(name={}, url={})".format(self.name, self.url)
+
 
 is_section_substitution = re.compile(r"{\[[^{}\s]+\]\S+?}").match
 """Check value matches substitution form of referencing value from other section.
