@@ -1,6 +1,8 @@
 """In case the tox environment is not correctly setup provision it and delegate execution"""
 from __future__ import absolute_import, unicode_literals
 
+import os
+
 from tox.exception import InvocationError
 
 
@@ -9,7 +11,9 @@ def provision_tox(provision_venv, args):
     with provision_venv.new_action("provision") as action:
         provision_args = [str(provision_venv.envconfig.envpython), "-m", "tox"] + args
         try:
-            action.popen(provision_args, redirect=False, report_fail=False)
+            env = os.environ.copy()
+            env[str("TOX_PROVISION")] = str("1")
+            action.popen(provision_args, redirect=False, report_fail=False, env=env)
             return 0
         except InvocationError as exception:
             return exception.exit_code
