@@ -13,7 +13,9 @@ import tox
 from tox import reporter
 from tox.action import Action
 from tox.config.parallel import ENV_VAR_KEY as PARALLEL_ENV_VAR_KEY
+from tox.constants import PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX
 from tox.package.local import resolve_package
+from tox.util.lock import get_unique_file
 from tox.util.path import ensure_empty_dir
 
 from .config import DepConfig
@@ -113,6 +115,7 @@ class VirtualEnv(object):
         self.popen = popen
         self._actions = []
         self.env_log = env_log
+        self._result_json_path = None
 
     def new_action(self, msg, *args):
         config = self.envconfig.config
@@ -130,6 +133,14 @@ class VirtualEnv(object):
             self.envconfig.envpython,
         )
 
+    def get_result_json_path(self):
+        if self._result_json_path is None:
+            if self.envconfig.config.option.resultjson:
+                self._result_json_path = get_unique_file(
+                    self.path, PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX
+                )
+        return self._result_json_path
+
     @property
     def hook(self):
         return self.envconfig.config.pluginmanager.hook
@@ -646,7 +657,6 @@ def prepend_shebang_interpreter(args):
     return args
 
 
-NO_DOWNLOAD = False
 _SKIP_VENV_CREATION = os.environ.get("_TOX_SKIP_ENV_CREATION_TEST", False) == "1"
 
 
@@ -658,7 +668,7 @@ def tox_testenv_create(venv, action):
         args.append("--system-site-packages")
     if venv.envconfig.alwayscopy:
         args.append("--always-copy")
-    if NO_DOWNLOAD:
+    if not venv.envconfig.download:
         args.append("--no-download")
     # add interpreter explicitly, to prevent using default (virtualenv.ini)
     args.extend(["--python", str(config_interpreter)])
